import os
from configparser import ConfigParser
from pathlib import Path


class MastodonConfiguration():

    def __init__(self, application_name):
        try:
            self.application_name = application_name
            self.base_config_folder = os.path.expanduser("~"+"/.config/"+application_name)
            Path(self.base_config_folder).mkdir(parents=True, exist_ok=True)
            self.config_file = self.base_config_folder+"/"+application_name+".ini"

            self.config = ConfigParser()
            if not os.path.exists(self.config_file):
                self.config[application_name] = {}
                self.config[application_name]["server"] = "social.example.com"
                self.config[application_name]["client_id"] = ""
                self.config[application_name]["client_secret"] = ""
                with open(self.config_file, "w") as configfile:
                    self.config.write(configfile)
            self.config.read(self.config_file)
        except Exception as e:
            print(e)

    def get_value(self, key) -> str:
        try:
            return self.config.get(self.application_name, key)
        except Exception as e:
            return ""

    def set_value(self, key, value) -> str:
        self.config[self.application_name][key] = value
        return value

    def save(self):
        with open(self.config_file, "w") as configfile:
            self.config.write(configfile)

